// Copyright (c) 2022. Lightricks. All rights reserved.
// Created by Nir Moshe.

var attachmentsPromise = null;
const FIELDS = new Map();

/**
 * Main entrypoint. This a heuristic, the document should be ready by now!
 * Note: that simple solutions like `document.addEventListener("DOMContentLoaded", ...)` simply 
 * doesnt work in Webview.
 */
 setTimeout(init, 200);


function init() {
    executeWithAndroidReporting(() => cleanAllFields());

    const zendeskFilesUploader = new ZendeskFilesUploader(
        AndroidInformationController.tokensURL(),
        AndroidInformationController.staticFilesServerURL()
    );

    // Hide the attachments section.
    executeWithAndroidReporting(() => hideAttachmentsSection());

    // Name of the application.
    executeWithAndroidReporting(() => FIELDS.set('6963575926418', AndroidInformationController.applicationName()));

    // Device type.
    executeWithAndroidReporting(() => FIELDS.set('6963756766098', AndroidInformationController.deviceType()));

    // App Version
    executeWithAndroidReporting(() => FIELDS.set('7164528653074', AndroidInformationController.applicationVersion()));

    // iOS / OS version
    executeWithAndroidReporting(() => FIELDS.set('6963799767186', AndroidInformationController.osVersion()));

    // Installation ID
    executeWithAndroidReporting(() => FIELDS.set('6963770579218', AndroidInformationController.installationId()));

    // LTID
    executeWithAndroidReporting(() => FIELDS.set('6963803775762', AndroidInformationController.ltId()));

    // Available storage space (in GB)
    executeWithAndroidReporting(() => FIELDS.set('6963971498770', AndroidInformationController.availableStorageSpace()));

    // Country
    executeWithAndroidReporting(() => FIELDS.set('6963964527378', AndroidInformationController.country()));

    // Platform
    executeWithAndroidReporting(() => FIELDS.set('8293121130130', AndroidInformationController.platform()));

    // Android app installer.
    executeWithAndroidReporting(() => FIELDS.set('8549093919122', AndroidInformationController.installerIsNotGooglePlay()));

    // Hide all the fields.
    FIELDS.forEach((_, fieldNumber) => {
        executeWithAndroidReporting(() => hideCustomField(fieldNumber))
    });

    // override the behaviour of the submit button.
    executeWithAndroidReporting(() => document.getElementById('new_request').addEventListener('submit', postEnrichedForm));

    // Start uploading the files...
    executeWithAndroidReporting(() => {
        let files = [];
        for (let i = 0; i < AndroidInformationController.attachmentsSize(); i++) {
            files.push(
                zendeskFile(AndroidInformationController.attachmentName(i),
                            AndroidInformationController.attachmentContent(i))
            );
        }
        attachmentsPromise = uploadMultipleFilesToZendesk(files, zendeskFilesUploader);
    });
}

/**
 * Executes the given function with custom exception handling - it catches all exceptions 
 * and notify the Android layer.
 * 
 * @param {Function} fn function to decorate.
 */
async function executeWithAndroidReporting(fn) {
    const f = async function() {
        try {
            fn.apply(this, arguments);
        } catch (err) {
            AndroidInformationController.reportJSError(`${err}: ${err.stack}`);
        }      
    };

    await f();
}

async function postEnrichedForm() {
    executeWithAndroidReporting(async () => {
        let attachments = await attachmentsPromise;
        attachments.forEach((attachment) => {
            let fileAttachment = JSON.stringify(attachment);
            createFileField(fileAttachment);
        });
        FIELDS.forEach((value, fieldNumber) => createField(fieldNumber, value));
    });

    return true;
}

/**
 * In case the user try to submit the form without filling all the required fields then Zendesk
 * will send the user to the URL: https://lightricks.zendesk.com/hc/en-us/requests.
 * In this case the attachments from the previous execution are still exists in the form, so
 * basically we are trying to remove all these attachments.
 */
function cleanAllRelevantFields() {
    let inputsToRemove = document.getElementsByName('request[attachments][]');
    for (let i = 0; i < inputsToRemove.length; i++) {
        inputsToRemove[i].remove();
    }
}

function createFileField(fileAttachment) {
    const element = document.createElement('input');
    element.setAttribute('name', 'request[attachments][]');
    element.setAttribute('type', 'hidden');
    element.value = fileAttachment;
    document.getElementById('new_request').appendChild(element);
}

function createField(number, value) {
    const element = document.createElement("input");
    element.type = "hidden";
    element.name = `request[custom_fields][${number}]`;
    element.value = value;
    document.getElementById('new_request').appendChild(element);
}

function hideCustomField(number) {
    let collection = document.getElementsByClassName(`request_custom_fields_${number}`);
    if (collection.length == 0) {
        throw new Error(`Can't find field request_custom_fields_${number}`);
    }

    collection[0].style.display = "none";
}

function hideAttachmentsSection() {
    let dropzone = document.getElementById("upload-dropzone");
    if (dropzone == undefined) {
        throw new Error("Can't find field upload-dropzone");
    }

    dropzone.parentNode.style.display = "none";
}

/**
 * Uploads multiple files into Zendesk.
 * 
 * @param {[{name: "string", content: "string"}]} filesArray represents list of files.
 * @param {ZendeskFilesUploader} zendeskFilesUploader 
 * @returns {Promise} object which is satisfied when all files were successfuly sent.
 */
 function uploadMultipleFilesToZendesk(filesArray, zendeskFilesUploader) {    
    return Promise.all(filesArray.map((zf) => zendeskFilesUploader.uploadFileToZendesk(zf.name, zf.content)));
}

function zendeskFile(_name, _content) {
    return {
        name: _name,
        content: _content
    };
}

/**
 * This object allows you to simply upload static file into Zendesk system.
 * 
 * @param {string} tokensURL Zendesk URL for fetching new tokens.
 * @param {string} staticFilesServerURL Zendesk URL for storing static files.
 */
 function ZendeskFilesUploader(tokensURL, staticFilesServerURL) {
    const token = (async () => await fetchToken())();

    async function fetchToken() {
        const response = await fetch(tokensURL);
        if (!response.ok) {
            throw new Error(`Failed fetching token...${response}`);
        }
    
        const obj = await response.json();
        return obj['current_session']['csrf_token'];
    };

    /**
     * Upload a new file asynchronously into Zendesk system. The function 
     * 
     * @param {string} filename The filename which is going to be presented in Zendesk dashboard.
     * @param {string} content The content of the file which is going to be presented in Zendesk dashboard.
     * 
     * @returns {object} The object that returns by Zendesk static files server ( @see staticFilesServerURL ).
     *  You should use this object by embbeding it into "Zendesk's new ticket form", before submitting it.
     */
    this.uploadFileToZendesk = async function(filename, content) {
        var blob = new Blob([content]);
        var formData = new FormData();
        formData.append('file', blob, filename);
        formData.append('authenticity_token', await token);
        const resp = await fetch(staticFilesServerURL, {
            method: 'POST',
            body: formData
        });

        return await resp.json();
    };
}
